<?php

namespace Pheanstalk\Exception;

use Pheanstalk\Exception;

/**
 * An exception originating as a beanstalkd server error.
 *
 * @author  Paul Annesley
 */
class ServerException extends Exception
{
}
